#include <Arduino.h>
#include <LedControl.h>
#include <Adafruit_MCP23X17.h>

/*
   pin 11 is connected to the DataIn
   pin 13 is connected to the CLK
   pin 10 is connected to LOAD
   2 MX7219
*/
LedControl lc = LedControl(11, 13, 10, 2);

/* 2 MCP23017
   conencted via i2c
*/
Adafruit_MCP23X17 mcp1;
Adafruit_MCP23X17 mcp2;

/*
   Print a number on the 7 segment display(s)
   disp 0: result: controller 1, digit 0,1,2,3
   disp 1: data in: controller 2, digit 0,1,2,3
   disp 2: register: controller 2, digit 4,5,6,7
   disp 3: select: controller 1, digit 4
   disp 4: carry: controller 1, led 5(a)
   v: value
     result, data in, register: 8bit number
     select: 3bit number
     carry: =0: LED off !=0 LED on
*/
void printNumber(int disp, int v) {
  int ones;
  int tens;
  int hundreds;
  boolean negative = false;
  int c; //controller
  int o; //offset for digit

  switch (disp) {
    case 0: c = 0; o = 0; break;
    case 1: c = 1; o = 0; break;
    case 2: c = 1; o = 4; break;
    case 3: c = 0; o = 4; break;
    case 4: c = 0; o = 5; break;
    default: return;
  }

  if (v < 0) {
    negative = true;
    v = v * -1;
  }
  ones = v % 10;
  v = v / 10;
  tens = v % 10;
  v = v / 10;
  hundreds = v;

  if (disp < 3) { //result, data in, register (-127..126 or 0..255)
    if (negative) {
      //print character '-' in the leftmost column
      lc.setChar(c, 0 + o, '-', false);
    }
    else {
      //print a blank in the sign column
      lc.setChar(c, 0 + o, ' ', false);
    }
    //Now print the number digit by digit
    lc.setDigit(c, 1 + o, (byte)hundreds, false);
    lc.setDigit(c, 2 + o, (byte)tens, false);
    lc.setDigit(c, 3 + o, (byte)ones, false);
  }
  else if (disp == 3) { //select 0..7
    lc.setDigit(c, 0 + o, (byte)ones, false);
  }
  else if (disp == 4) {
    if (ones == 0) {
      lc.setChar(c, 0 + o, ' ', false);
    }
    else {
      lc.setChar(c, 0 + o, '8', false);
    }
  }
}

void setup() {
  Serial.begin(9600);
  while (!Serial);
  Serial.println("RelayComp Display");

  /*  The MAX72XX is in power-saving mode on startup, we have to do a wakeup call */
  lc.shutdown(0, false);
  lc.shutdown(1, false);
  /* Set the brightness */
  lc.setIntensity(0, 8);
  lc.setIntensity(1, 8);
  /* clear the display */
  lc.clearDisplay(0);
  lc.clearDisplay(1);

  /* Start mcps */
  if (!mcp1.begin_I2C(0x20)) {
    Serial.println("Error.");
    while (1);
  }
  if (!mcp2.begin_I2C(0x21)) {
    Serial.println("Error.");
    while (1);
  }
  /* configure pins for input */
  for (int i = 0; i < 16; i++) {
    mcp1.pinMode(i, INPUT);
    mcp2.pinMode(i, INPUT);
  }

  /* signed/unsigned switch */
  pinMode(3, INPUT);
}


void loop() {
  uint8_t res;
  uint8_t inp;
  uint8_t reg;
  uint8_t sel;
  uint8_t carry;
  uint8_t s;

  res = mcp2.readGPIO(0);
  inp = mcp1.readGPIO(0);
  reg = mcp2.readGPIO(1);
  sel = mcp1.readGPIO(1) & 7;
  carry = (mcp1.readGPIO(1) & 8) >> 3;
  s = digitalRead(3);

  if (s == 0) {
    printNumber(0, res);
    printNumber(1, inp);
    printNumber(2, reg);
  }
  else {
    printNumber(0, (int8_t)res);
    printNumber(1, (int8_t)inp);
    printNumber(2, (int8_t)reg);
  }
  printNumber(3, sel);
  printNumber(4, carry);
}
